﻿//AddPathOperationsToLayoutMenu.jsx
//An InDesign JavaScript by SPI
//
//Adds common path drawing commands to the layout context menu.
//
//by Olav Martin Kvern
//Distributed by Silicon Publishing, Inc.
//
//Our web site says, "Silicon Publishing, Inc. provides electronic publishing solutions and customizations 
//that automate the distribution of information between multiple sources and destinations." But what does
//that really mean? We make custom software for publishing. Some of it is for intense, data-driven, high-
//volume applications--think: directories, catalogs, and customized itineraries. Some of it is for web-to-print 
//or print-to-web applications--think: business cards, flyers, brochures, data sheets. Some of it is none of the 
//above. Use your imagination--if there's some chance that we can make your work easier, drop us a line! 
//sales@siliconpublishing.com
//
//Because this script uses event listeners, we have to specify a persistent script engine.
//The "session" engine exists as long as InDesign is running.
#targetengine "session"

//Make sure that user interaction is turned on.
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL;

var appFolder = null;
var addPointsScript = null;
var cornerEffectsScript = null;
main();
function main(){
	appFolder = app.filePath;
	if((appFolder.exists) && (appFolder != null)){
		addPointsScript = getScriptFile ("AddPoints.jsx");
		cornerEffectsScript = getScriptFile("CornerEffects.jsx");
	}
	customizeLayoutMenu();
}
function customizeLayoutMenu(){
	//The locale-independent name (aka "key string") for the 
	//Layout context menu is "$ID/RtMouseLayout".
	var layoutContextMenu = app.menus.item("$ID/RtMouseLayout");
	if(layoutContextMenu.submenus.itemByName("Path").isValid){
		layoutContextMenu.submenus.itemByName("Path").remove();
	}
	var menuSeparator = layoutContextMenu.menuSeparators.item(0);
	var pathSubmenu = layoutContextMenu.submenus.add("Path", LocationOptions.AFTER, menuSeparator);
	//If the AddPoints.jsx script was not found, do not add the menu item.
	if(addPointsScript != null){
		var addPointsScriptMenuAction = app.scriptMenuActions.add("Add Points");
		var addPointsEventListener = addPointsScriptMenuAction.eventListeners.add("onInvoke", handleAddPoints);
		pathSubmenu.menuItems.add(addPointsScriptMenuAction);
	}
	//If the CornerEffects.jsx script was not found, do not add the menu item.
	if(cornerEffectsScript != null){
		var cornerEffectsScriptMenuAction = app.scriptMenuActions.add("Corner Effects");
		var cornerEffectsEventListener = cornerEffectsScriptMenuAction.eventListeners.add("onInvoke", handleCornerEffects);
		pathSubmenu.menuItems.add(cornerEffectsScriptMenuAction);
	}
	//Add the menu items for the actions that are usually buried at 
	//the bottom of the Object menu.
	var menuAction = app.menuActions.item("$ID/Close Path");
	pathSubmenu.menuItems.add(menuAction);
	menuAction = app.menuActions.item("$ID/Open Path");
	pathSubmenu.menuItems.add(menuAction);
	menuAction = app.menuActions.item("$ID/Reverse Path");
	pathSubmenu.menuItems.add(menuAction);
	menuAction = app.menuActions.item("$ID/Join");
	pathSubmenu.menuItems.add(menuAction);
	menuAction = app.menuActions.item("$ID/Rectangle");
	pathSubmenu.menuItems.add(menuAction);	
	menuAction = app.menuActions.item("$ID/Ellipse");
	pathSubmenu.menuItems.add(menuAction);	
	menuAction = app.menuActions.item("$ID/Add");
	pathSubmenu.menuItems.add(menuAction);	
	menuAction = app.menuActions.item("$ID/Subtract");
	pathSubmenu.menuItems.add(menuAction);	
	menuAction = app.menuActions.item("$ID/Intersect");
	pathSubmenu.menuItems.add(menuAction);	
	menuAction = app.menuActions.item("$ID/Minus Back");
	pathSubmenu.menuItems.add(menuAction);	
	menuAction = app.menuActions.item("$ID/Exclude Overlap");
	pathSubmenu.menuItems.add(menuAction);	
	menuAction = app.menuActions.item("$ID/Make Compound Path");
	pathSubmenu.menuItems.add(menuAction);
	menuAction = app.menuActions.item("$ID/Release Compound Path");
	pathSubmenu.menuItems.add(menuAction);
	layoutContextMenu.menuSeparators.add(LocationOptions.AFTER, pathSubmenu, {label:"PathSubmenuSeparator"});
}
function handleAddPoints(){
	app.doScript(addPointsScript, ScriptLanguage.JAVASCRIPT, undefined, UndoModes.ENTIRE_SCRIPT);
}
function handleCornerEffects(){
	app.doScript(cornerEffectsScript, ScriptLanguage.JAVASCRIPT, undefined, UndoModes.ENTIRE_SCRIPT);
}
function getScriptFile(fileName){
	var result = null;
	if(appFolder != null){
		var scriptsFolder = Folder(appFolder.fullName + "/Scripts/Scripts Panel");
		if(scriptsFolder.exits == false){
			return result;
		}
		var samplesFolder = Folder(scriptsFolder.fullName + "/Samples");
		if(samplesFolder.exists == false){
			return result;
		}
		var javascriptFolder = Folder(samplesFolder.fullName + "/JavaScript");
		if(javascriptFolder.exists == false){
			return result;
		}
		var script = File(javascriptFolder.fullName + "/" + fileName);
		if(script.exists != null){
			result = script;
		}
	}	
	return result;
}