﻿//IterativePathCuts.jsx
//An InDesign JavaScript by SPI
//
//Converts the selected InDesign paths to g-code for use in a CNC ("Computer Numeric Control"--basically, a
//computer controlled router. To use the output files (which you should give the extension ".nc"), you'll
//need some kind of machine control software, such as MACH 3.
//
//by Olav Martin Kvern
//Distributed by Silicon Publishing, Inc.
//
//Our web site says, "Silicon Publishing, Inc. provides electronic publishing solutions and customizations 
//that automate the distribution of information between multiple sources and destinations." But what does
//that really mean? We make custom software for publishing. Some of it is for intense, data-driven, high-
//volume applications--think: directories, catalogs, and customized itineraries. Some of it is for web-to-print 
//or print-to-web applications--think: business cards, flyers, brochures, data sheets. Some of it is none of the 
//above. Use your imagination--if there's some chance that we can make your work easier, drop us a line! 
//sales@siliconpublishing.com
//
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL;
main();
function main(){
	var objectList = new Array;
	if(app.documents.length > 0){
		if(app.selection.length > 0){
			//Set measurement units.
			app.documents.item(0).viewPreferences.horizontalMeasurementUnits = MeasurementUnits.MILLIMETERS;
			app.documents.item(0).viewPreferences.verticalMeasurementUnits = MeasurementUnits.MILLIMETERS;
			var zeroPoint = app.documents.item(0).zeroPoint;
			for(var counter = 0; counter < app.selection.length; counter++){
				switch(app.selection[counter].constructor.name){
					case "Rectangle":
					case "Oval":
					case "Polygon":
					case "GraphicLine":
						objectList.push(app.selection[counter]);
						break;
				}
			}
			if(objectList.length > 0){
				displayDialog(objectList);
			}
		}
	}
}
function displayDialog(objectList){
	var labelWidth = 60;
	var mmWidth = 20;
    var dialog = app.dialogs.add({name:"Convert Path to GCode"});
    with(dialog.dialogColumns.add()){
        with(dialogRows.add()){	
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Start Z:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var startZField = realEditboxes.add({editValue:6});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"mm", minWidth:mmWidth});
			}
		}
        with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Start X:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var startXField = realEditboxes.add({editValue:0});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"mm", minWidth:mmWidth});
			}
		}
        with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Start Y:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var startYField = realEditboxes.add({editValue:0});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"mm", minWidth:mmWidth});
			}
		}
        with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"End Z:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var endZField = realEditboxes.add({editValue:0});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"mm", minWidth:mmWidth});
			}
		}
        with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Interim Z:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var interimZField = realEditboxes.add({editValue:8});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"mm", minWidth:mmWidth});
			}
		}
        with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Step:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var stepField = realEditboxes.add({editValue:1});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"mm", minWidth:mmWidth});
			}
		}	
        with(dialogRows.add()){
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"Speed:", minWidth:labelWidth});
			}
			with(dialogColumns.add()){
				var speedField = realEditboxes.add({editValue:250});
			}
			with(dialogColumns.add()){
				staticTexts.add({staticLabel:"mm", minWidth:mmWidth});
			}
		}	
        with(dialogRows.add()){
			var addPauseCheckbox = checkboxControls.add({staticLabel:"Pause between objects at end", checkedState:false});
		}	
	}
	var result = dialog.show();
	if(result == true){
		var step = stepField.editValue;
		var startZ = startZField.editValue;
		var endZ = endZField.editValue;
		var interimZ = interimZField.editValue;
		var startX = startXField.editValue;
		var startY = startYField.editValue;
		var speed = speedField.editValue;
		var addPause = addPauseCheckbox.checkedState;
		dialog.destroy();
		convertPathsToGCode(objectList, step, startZ, endZ, startX, startY, interimZ, addPause, speed);
	}
	else{
		dialog.destroy();
	}
}
function convertPathsToGCode(objectList, step, startZ, endZ, startX, startY, interimZ, addPause, speed){
	var pageItem;
	//Display a save file dialog box.
	var file = File.saveDialog("Save gCode File as:");
	var numberOfIterations = Math.round((startZ - endZ)/step);
	if((file != null)&&(file != "")){
		file.lineFeed = "Windows";
		//Open the file and add job header info.
		file.open("w");
		try{
			file.writeln(";File: " + app.documents.item(0).filePath + "\r");
		}
		catch(error){
			file.writeln(";File: Untitled\r");
		}
		var date = Date();
		file.writeln(";" + date);
		file.writeln(";Speed\rf " + speed.toString() + "\r\r");
		//Lift and move to initial position.
		file.writeln(";Move to starting position.\rg0 z 30\rg0 x 0 y 0\r");
		//Iteration works like this: per step, per object, and then per path. That way,
		//fewer objects are completely cut out before the end of the job.
		for(var counter = 0; counter <= numberOfIterations; counter++){
			file.writeln(";Pass: " + counter + "\r");
			if(counter == numberOfIterations){
				currentStep = endZ;
			}
			else{
				currentStep = startZ - (step * counter);
			}
			for(var pageItemCounter = 0; pageItemCounter < objectList.length; pageItemCounter++){		
				file.writeln(";Page Item: " + pageItemCounter + "\r");
				pageItem = objectList[pageItemCounter];
				for(var pathCounter = 0; pathCounter < pageItem.paths.length; pathCounter++){
					convertToGCode(file, pageItem.paths.item(pathCounter), currentStep, startX, startY, interimZ);
				}
				if((addPause == true)&&(counter == numberOfIterations)){
					file.writeln(";Pause\rm00\r;");
				}
			}
		}
		//Lift and move to final position.
		file.writeln("g0 z 30\rg0 x 0 y 0\r");
		alert("Done!");
	}
}
function convertToGCode(file, path, currentStep, startX, startY, interimZ){
	var x, y;
	var pathPoints = path.pathPoints;
	file.writeln(";New path\r");
	for(var pointCounter = 0; pointCounter < pathPoints.length; pointCounter++){
		x = startX + truncate(pathPoints.item(pointCounter).anchor[0]);
		//InDesign vertical axis is upside down, so invert.
		y = startY + truncate(-1*(pathPoints.item(pointCounter).anchor[1]));
		if(pointCounter == 0){
			//Move to the first point on the path at engine speed.
			file.writeln("g0 x " + x + " y " + y);
			//Lower the router to the current z location.
			file.writeln(";\r;Lower router\rg1 z " + currentStep + "\r");
		}
		else{
			//Move the router to the next point on the path.
			file.writeln("g1 x " + x + " y " + y);
		}
	}
	//Back to the start.
	if(path.pathType == PathType.CLOSED_PATH){
		x = startX + truncate(pathPoints.item(0).anchor[0]);
		//InDesign vertical axis is upside down, so invert.
		y = startY + truncate(-1*(pathPoints.item(0).anchor[1]));
		file.writeln(";Close path\rg1 x " + x + " y " + y);
	}
	file.writeln(";End of path");
	//Lift to iterim and pause.
	file.writeln("g0 z " + interimZ + "\r;m00\r;");
}
function truncate(number){
	number = (Math.round(number * 10000))/10000;
	return number;
}